#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import sys
import tempfile
import time
from pathlib import Path

import pyLSV2

logging.basicConfig(level=logging.DEBUG)

if __name__ == "__main__":
    address = '192.168.56.101'
    if len(sys.argv) > 1:
        address = sys.argv[1]

    print('Connecting to {}'.format(address))

    con = pyLSV2.LSV2(address, port=19000, timeout=5, safe_mode=False)
    con.connect()

    print('Connected to "{Control}" with NC Software "{NC_Version}"'.format(**con.get_versions()))
    print('Running LSV2 Version {LSV2_Version} with Flags {LSV2_Version_Flags}'.format(**con.get_system_parameter()))

    print('Drive Info: {}'.format(con.get_drive_info()))
    print('Current Folder {Path} Free Space: {Free Size} Attrib: {Dir_Attributs}'.format(**con.get_directory_info()))

    print('PLC Marker: {}'.format(con.read_plc_memory(address=0, mem_type=pyLSV2.PLC_MEM_TYPE_MARKER, count=15)))
    print('PLC Word: {}'.format(con.read_plc_memory(address=6, mem_type=pyLSV2.PLC_MEM_TYPE_WORD, count=10)))
    print('PLC Double Word: {}'.format(con.read_plc_memory(address=0, mem_type=pyLSV2.PLC_MEM_TYPE_DWORD, count=10)))
    print('PLC String: {}'.format(con.read_plc_memory(address=2, mem_type=pyLSV2.PLC_MEM_TYPE_STRING, count=2)))
    print('PLC Input: {}'.format(con.read_plc_memory(address=0, mem_type=pyLSV2.PLC_MEM_TYPE_INPUT, count=5)))
    print('PLC Word Output: {}'.format(con.read_plc_memory(address=10, mem_type=pyLSV2.PLC_MEM_TYPE_OUTPUT_WORD, count=5)))

    # reading of machine parameter for old an new style names
    if not con.is_itnc():
        # new stype
        print('Current Language: {}'.format(con.get_machine_parameter('CfgDisplayLanguage.ncLanguage')))
    else:
        # old style
        print('Current Language: {}'.format(con.get_machine_parameter('7230.0')))
    
    # changing the value of a machine parameter
    #con.login(pyLSV2.LOGIN_PLCDEBUG)
    #new style: con.set_machine_parameter('CfgDisplayLanguage.ncLanguage', 'CZECH', safe_to_disk=False)
    #old style: con.set_machine_parameter('7230.0', '2', safe_to_disk=False)
    #con.logout(pyLSV2.LOGIN_PLCDEBUG)

    # read program stack
    print('Current Program Stack: {}'.format(con.get_program_stack()))

    # demo for sending key codes
    con.login(pyLSV2.LOGIN_MONITOR)
    con.set_keyboard_access(False)
    con.send_key_code(pyLSV2.KEY_MODE_MANUAL)
    time.sleep(3)
    con.send_key_code(pyLSV2.KEY_MODE_PGM_EDIT)
    con.set_keyboard_access(True)
    con.logout(pyLSV2.LOGIN_MONITOR)

    # demo for reading the current tool with fallback if it is not supported by control
    t_info = con.get_spindle_tool_status()
    if t_info is not False:
        print('Current tool number {Number}.{Index} with Axis {Axis} Length {Length} Radius {Radius}'.format(**t_info))
    else:
        print('Direct reading to current tool not supported for this control type. using backup strategy')
        if con.is_tnc():
            pocket_table_path = 'TNC:/table/tool_p.tch'
            transfer_binary = True
            spindel_lable = '0.0'
        elif con.is_itnc():
            pocket_table_path = 'TNC:/TOOL_P.TCH'
            transfer_binary = False
            spindel_lable = '0'
        else:
            pocket_table_path = 'TNC:/table/ToolAllo.tch'
            transfer_binary = True
            spindel_lable = '0.0'

        with tempfile.TemporaryDirectory(suffix=None, prefix='pyLSV2_') as tmp_dir_name:
            local_recive_path = Path(tmp_dir_name).joinpath('tool_p.tch')
            con.recive_file(local_path=str(local_recive_path), remote_path=pocket_table_path, binary_mode=transfer_binary)
            tr = pyLSV2.TableReader()
            pockets = tr.parse_table(local_recive_path)
            spindel = list(filter(lambda pocket: pocket['P'] == spindel_lable, pockets))[0]
            print('Current tool number {T} with name {TNAME}'.format(**spindel))

    con.disconnect()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""error code definitions and decoding, translation of status information into readable text"""
import gettext
import os
import warnings

from . import const as L_C

# Error map
LSV2_ERROR_T_ER_BAD_FORMAT = 20
LSV2_ERROR_T_ER_UNEXPECTED_TELE = 21
LSV2_ERROR_T_ER_UNKNOWN_TELE = 22
LSV2_ERROR_T_ER_NO_PRIV = 23
LSV2_ERROR_T_ER_WRONG_PARA = 24
LSV2_ERROR_T_ER_BREAK = 25
LSV2_ERROR_T_ER_BAD_KEY = 30
LSV2_ERROR_T_ER_BAD_FNAME = 31
LSV2_ERROR_T_ER_NO_FILE = 32
LSV2_ERROR_T_ER_OPEN_FILE = 33
LSV2_ERROR_T_ER_FILE_EXISTS = 34
LSV2_ERROR_T_ER_BAD_FILE = 35
LSV2_ERROR_T_ER_NO_DELETE = 36
LSV2_ERROR_T_ER_NO_NEW_FILE = 37
LSV2_ERROR_T_ER_NO_CHANGE_ATT = 38
LSV2_ERROR_T_ER_BAD_EMULATEKEY = 39
LSV2_ERROR_T_ER_NO_MP = 40
LSV2_ERROR_T_ER_NO_WIN = 41
LSV2_ERROR_T_ER_WIN_NOT_AKTIV = 42
LSV2_ERROR_T_ER_ANZ = 43
LSV2_ERROR_T_ER_FONT_NOT_DEFINED = 44
LSV2_ERROR_T_ER_FILE_ACCESS = 45
LSV2_ERROR_T_ER_WRONG_DNC_STATUS = 46
LSV2_ERROR_T_ER_CHANGE_PATH = 47
LSV2_ERROR_T_ER_NO_RENAME = 48
LSV2_ERROR_T_ER_NO_LOGIN = 49
LSV2_ERROR_T_ER_BAD_PARAMETER = 50
LSV2_ERROR_T_ER_BAD_NUMBER_FORMAT = 51
LSV2_ERROR_T_ER_BAD_MEMADR = 52
LSV2_ERROR_T_ER_NO_FREE_SPACE = 53
LSV2_ERROR_T_ER_DEL_DIR = 54
LSV2_ERROR_T_ER_NO_DIR = 55
LSV2_ERROR_T_ER_OPERATING_MODE = 56
LSV2_ERROR_T_ER_NO_NEXT_ERROR = 57
LSV2_ERROR_T_ER_ACCESS_TIMEOUT = 58
LSV2_ERROR_T_ER_NO_WRITE_ACCESS = 59
LSV2_ERROR_T_ER_STIB = 60
LSV2_ERROR_T_ER_REF_NECESSARY = 61
LSV2_ERROR_T_ER_PLC_BUF_FULL = 62
LSV2_ERROR_T_ER_NOT_FOUND = 63
LSV2_ERROR_T_ER_WRONG_FILE = 64
LSV2_ERROR_T_ER_NO_MATCH = 65
LSV2_ERROR_T_ER_TOO_MANY_TPTS = 66
LSV2_ERROR_T_ER_NOT_ACTIVATED = 67
LSV2_ERROR_T_ER_DSP_CHANNEL = 70
LSV2_ERROR_T_ER_DSP_PARA = 71
LSV2_ERROR_T_ER_OUT_OF_RANGE = 72
LSV2_ERROR_T_ER_INVALID_AXIS = 73
LSV2_ERROR_T_ER_STREAMING_ACTIVE = 74
LSV2_ERROR_T_ER_NO_STREAMING_ACTIVE = 75
LSV2_ERROR_T_ER_TO_MANY_OPEN_TCP = 80
LSV2_ERROR_T_ER_NO_FREE_HANDLE = 81
LSV2_ERROR_T_ER_PLCMEMREMA_CLEAR = 82
LSV2_ERROR_T_ER_OSZI_CHSEL = 83
LSV2_ERROR_LSV2_BUSY = 90
LSV2_ERROR_LSV2_X_BUSY = 91
LSV2_ERROR_LSV2_NOCONNECT = 92
LSV2_ERROR_LSV2_BAD_BACKUP_FILE = 93
LSV2_ERROR_LSV2_RESTORE_NOT_FOUND = 94
LSV2_ERROR_LSV2_DLL_NOT_INSTALLED = 95
LSV2_ERROR_LSV2_BAD_CONVERT_DLL = 96
LSV2_ERROR_LSV2_BAD_BACKUP_LIST = 97
LSV2_ERROR_LSV2_UNKNOWN_ERROR = 99
LSV2_ERROR_T_BD_NO_NEW_FILE = 100
LSV2_ERROR_T_BD_NO_FREE_SPACE = 101
LSV2_ERROR_T_BD_FILE_NOT_ALLOWED = 102
LSV2_ERROR_T_BD_BAD_FORMAT = 103
LSV2_ERROR_T_BD_BAD_BLOCK = 104
LSV2_ERROR_T_BD_END_PGM = 105
LSV2_ERROR_T_BD_ANZ = 106
LSV2_ERROR_T_BD_WIN_NOT_DEFINED = 107
LSV2_ERROR_T_BD_WIN_CHANGED = 108
LSV2_ERROR_T_BD_DNC_WAIT = 110
LSV2_ERROR_T_BD_CANCELLED = 111
LSV2_ERROR_T_BD_OSZI_OVERRUN = 112
LSV2_ERROR_T_BD_FD = 200
LSV2_ERROR_T_USER_ERROR = 255


def get_error_text(error_type, error_code, language='en'):
    """Parse error type and error code and return the error message.

    :param int error_type: type of error code.
    :param int error_code: code of error message.
    :param str language: language code for message.
    :raise: NotImplementedError if error type is != 1
    :return: error message in selected language
    :rtype: str
    """
    warnings.warn('Deprecation Warning! The definition of the LSV2_ERROR_ constants was extracted from pyLSV.translate_mesages to pyLSV2. Definition in LSV2 will be removed in future versions')

    locale_path = os.path.dirname(__file__) + '/locales'
    translate = gettext.translation(
        'error_text', localedir=locale_path, languages=[language], fallback=True)
    _ = translate.gettext

    if error_type != 1:
        raise NotImplementedError('Unknown error type: %d' % error_type)
    return {L_C.LSV2_ERROR_T_ER_BAD_FORMAT: _('LSV2_ERROR_T_ER_BAD_FORMAT'),
            L_C.LSV2_ERROR_T_ER_UNEXPECTED_TELE: _('LSV2_ERROR_T_ER_UNEXPECTED_TELE'),
            L_C.LSV2_ERROR_T_ER_UNKNOWN_TELE: _('LSV2_ERROR_T_ER_UNKNOWN_TELE'),
            L_C.LSV2_ERROR_T_ER_NO_PRIV: _('LSV2_ERROR_T_ER_NO_PRIV'),
            L_C.LSV2_ERROR_T_ER_WRONG_PARA: _('LSV2_ERROR_T_ER_WRONG_PARA'),
            L_C.LSV2_ERROR_T_ER_BREAK: _('LSV2_ERROR_T_ER_BREAK'),
            L_C.LSV2_ERROR_T_ER_BAD_KEY: _('LSV2_ERROR_T_ER_BAD_KEY'),
            L_C.LSV2_ERROR_T_ER_BAD_FNAME: _('LSV2_ERROR_T_ER_BAD_FNAME'),
            L_C.LSV2_ERROR_T_ER_NO_FILE: _('LSV2_ERROR_T_ER_NO_FILE'),
            L_C.LSV2_ERROR_T_ER_OPEN_FILE: _('LSV2_ERROR_T_ER_OPEN_FILE'),
            L_C.LSV2_ERROR_T_ER_FILE_EXISTS: _('LSV2_ERROR_T_ER_FILE_EXISTS'),
            L_C.LSV2_ERROR_T_ER_BAD_FILE: _('LSV2_ERROR_T_ER_BAD_FILE'),
            L_C.LSV2_ERROR_T_ER_NO_DELETE: _('LSV2_ERROR_T_ER_NO_DELETE'),
            L_C.LSV2_ERROR_T_ER_NO_NEW_FILE: _('LSV2_ERROR_T_ER_NO_NEW_FILE'),
            L_C.LSV2_ERROR_T_ER_NO_CHANGE_ATT: _('LSV2_ERROR_T_ER_NO_CHANGE_ATT'),
            L_C.LSV2_ERROR_T_ER_BAD_EMULATEKEY: _('LSV2_ERROR_T_ER_BAD_EMULATEKEY'),
            L_C.LSV2_ERROR_T_ER_NO_MP: _('LSV2_ERROR_T_ER_NO_MP'),
            L_C.LSV2_ERROR_T_ER_NO_WIN: _('LSV2_ERROR_T_ER_NO_WIN'),
            L_C.LSV2_ERROR_T_ER_WIN_NOT_AKTIV: _('LSV2_ERROR_T_ER_WIN_NOT_AKTIV'),
            L_C.LSV2_ERROR_T_ER_ANZ: _('LSV2_ERROR_T_ER_ANZ'),
            L_C.LSV2_ERROR_T_ER_FONT_NOT_DEFINED: _('LSV2_ERROR_T_ER_FONT_NOT_DEFINED'),
            L_C.LSV2_ERROR_T_ER_FILE_ACCESS: _('LSV2_ERROR_T_ER_FILE_ACCESS'),
            L_C.LSV2_ERROR_T_ER_WRONG_DNC_STATUS: _('LSV2_ERROR_T_ER_WRONG_DNC_STATUS'),
            L_C.LSV2_ERROR_T_ER_CHANGE_PATH: _('LSV2_ERROR_T_ER_CHANGE_PATH'),
            L_C.LSV2_ERROR_T_ER_NO_RENAME: _('LSV2_ERROR_T_ER_NO_RENAME'),
            L_C.LSV2_ERROR_T_ER_NO_LOGIN: _('LSV2_ERROR_T_ER_NO_LOGIN'),
            L_C.LSV2_ERROR_T_ER_BAD_PARAMETER: _('LSV2_ERROR_T_ER_BAD_PARAMETER'),
            L_C.LSV2_ERROR_T_ER_BAD_NUMBER_FORMAT: _('LSV2_ERROR_T_ER_BAD_NUMBER_FORMAT'),
            L_C.LSV2_ERROR_T_ER_BAD_MEMADR: _('LSV2_ERROR_T_ER_BAD_MEMADR'),
            L_C.LSV2_ERROR_T_ER_NO_FREE_SPACE: _('LSV2_ERROR_T_ER_NO_FREE_SPACE'),
            L_C.LSV2_ERROR_T_ER_DEL_DIR: _('LSV2_ERROR_T_ER_DEL_DIR'),
            L_C.LSV2_ERROR_T_ER_NO_DIR: _('LSV2_ERROR_T_ER_NO_DIR'),
            L_C.LSV2_ERROR_T_ER_OPERATING_MODE: _('LSV2_ERROR_T_ER_OPERATING_MODE'),
            L_C.LSV2_ERROR_T_ER_NO_NEXT_ERROR: _('LSV2_ERROR_T_ER_NO_NEXT_ERROR'),
            L_C.LSV2_ERROR_T_ER_ACCESS_TIMEOUT: _('LSV2_ERROR_T_ER_ACCESS_TIMEOUT'),
            L_C.LSV2_ERROR_T_ER_NO_WRITE_ACCESS: _('LSV2_ERROR_T_ER_NO_WRITE_ACCESS'),
            L_C.LSV2_ERROR_T_ER_STIB: _('LSV2_ERROR_T_ER_STIB'),
            L_C.LSV2_ERROR_T_ER_REF_NECESSARY: _('LSV2_ERROR_T_ER_REF_NECESSARY'),
            L_C.LSV2_ERROR_T_ER_PLC_BUF_FULL: _('LSV2_ERROR_T_ER_PLC_BUF_FULL'),
            L_C.LSV2_ERROR_T_ER_NOT_FOUND: _('LSV2_ERROR_T_ER_NOT_FOUND'),
            L_C.LSV2_ERROR_T_ER_WRONG_FILE: _('LSV2_ERROR_T_ER_WRONG_FILE'),
            L_C.LSV2_ERROR_T_ER_NO_MATCH: _('LSV2_ERROR_T_ER_NO_MATCH'),
            L_C.LSV2_ERROR_T_ER_TOO_MANY_TPTS: _('LSV2_ERROR_T_ER_TOO_MANY_TPTS'),
            L_C.LSV2_ERROR_T_ER_NOT_ACTIVATED: _('LSV2_ERROR_T_ER_NOT_ACTIVATED'),
            L_C.LSV2_ERROR_T_ER_DSP_CHANNEL: _('LSV2_ERROR_T_ER_DSP_CHANNEL'),
            L_C.LSV2_ERROR_T_ER_DSP_PARA: _('LSV2_ERROR_T_ER_DSP_PARA'),
            L_C.LSV2_ERROR_T_ER_OUT_OF_RANGE: _('LSV2_ERROR_T_ER_OUT_OF_RANGE'),
            L_C.LSV2_ERROR_T_ER_INVALID_AXIS: _('LSV2_ERROR_T_ER_INVALID_AXIS'),
            L_C.LSV2_ERROR_T_ER_STREAMING_ACTIVE: _('LSV2_ERROR_T_ER_STREAMING_ACTIVE'),
            L_C.LSV2_ERROR_T_ER_NO_STREAMING_ACTIVE: _('LSV2_ERROR_T_ER_NO_STREAMING_ACTIVE'),
            L_C.LSV2_ERROR_T_ER_TO_MANY_OPEN_TCP: _('LSV2_ERROR_T_ER_TO_MANY_OPEN_TCP'),
            L_C.LSV2_ERROR_T_ER_NO_FREE_HANDLE: _('LSV2_ERROR_T_ER_NO_FREE_HANDLE'),
            L_C.LSV2_ERROR_T_ER_PLCMEMREMA_CLEAR: _('LSV2_ERROR_T_ER_PLCMEMREMA_CLEAR'),
            L_C.LSV2_ERROR_T_ER_OSZI_CHSEL: _('LSV2_ERROR_T_ER_OSZI_CHSEL'),
            L_C.LSV2_ERROR_LSV2_BUSY: _('LSV2_ERROR_LSV2_BUSY'),
            L_C.LSV2_ERROR_LSV2_X_BUSY: _('LSV2_ERROR_LSV2_X_BUSY'),
            L_C.LSV2_ERROR_LSV2_NOCONNECT: _('LSV2_ERROR_LSV2_NOCONNECT'),
            L_C.LSV2_ERROR_LSV2_BAD_BACKUP_FILE: _('LSV2_ERROR_LSV2_BAD_BACKUP_FILE'),
            L_C.LSV2_ERROR_LSV2_RESTORE_NOT_FOUND: _('LSV2_ERROR_LSV2_RESTORE_NOT_FOUND'),
            L_C.LSV2_ERROR_LSV2_DLL_NOT_INSTALLED: _('LSV2_ERROR_LSV2_DLL_NOT_INSTALLED'),
            L_C.LSV2_ERROR_LSV2_BAD_CONVERT_DLL: _('LSV2_ERROR_LSV2_BAD_CONVERT_DLL'),
            L_C.LSV2_ERROR_LSV2_BAD_BACKUP_LIST: _('LSV2_ERROR_LSV2_BAD_BACKUP_LIST'),
            L_C.LSV2_ERROR_LSV2_UNKNOWN_ERROR: _('LSV2_ERROR_LSV2_UNKNOWN_ERROR'),
            L_C.LSV2_ERROR_T_BD_NO_NEW_FILE: _('LSV2_ERROR_T_BD_NO_NEW_FILE'),
            L_C.LSV2_ERROR_T_BD_NO_FREE_SPACE: _('LSV2_ERROR_T_BD_NO_FREE_SPACE'),
            L_C.LSV2_ERROR_T_BD_FILE_NOT_ALLOWED: _('LSV2_ERROR_T_BD_FILE_NOT_ALLOWED'),
            L_C.LSV2_ERROR_T_BD_BAD_FORMAT: _('LSV2_ERROR_T_BD_BAD_FORMAT'),
            L_C.LSV2_ERROR_T_BD_BAD_BLOCK: _('LSV2_ERROR_T_BD_BAD_BLOCK'),
            L_C.LSV2_ERROR_T_BD_END_PGM: _('LSV2_ERROR_T_BD_END_PGM'),
            L_C.LSV2_ERROR_T_BD_ANZ: _('LSV2_ERROR_T_BD_ANZ'),
            L_C.LSV2_ERROR_T_BD_WIN_NOT_DEFINED: _('LSV2_ERROR_T_BD_WIN_NOT_DEFINED'),
            L_C.LSV2_ERROR_T_BD_WIN_CHANGED: _('LSV2_ERROR_T_BD_WIN_CHANGED'),
            L_C.LSV2_ERROR_T_BD_DNC_WAIT: _('LSV2_ERROR_T_BD_DNC_WAIT'),
            L_C.LSV2_ERROR_T_BD_CANCELLED: _('LSV2_ERROR_T_BD_CANCELLED'),
            L_C.LSV2_ERROR_T_BD_OSZI_OVERRUN: _('LSV2_ERROR_T_BD_OSZI_OVERRUN'),
            L_C.LSV2_ERROR_T_BD_FD: _('LSV2_ERROR_T_BD_FD'),
            L_C.LSV2_ERROR_T_USER_ERROR: _('LSV2_ERROR_T_USER_ERROR')}.get(error_code, _('LSV2_ERROR_UNKNOWN_CODE'))


def get_program_status_text(code, language='en'):
    """Translate status code of program state to text

    :param int code: status code of program state
    :param str language: optional. language code for translation of text
    :returns: readable text for execution state
    :rtype: str
    """
    locale_path = os.path.dirname(__file__) + '/locales'
    translate = gettext.translation(
        'message_text', localedir=locale_path, languages=[language], fallback=True)
    return {L_C.PGM_STATE_STARTED: translate.gettext('PGM_STATE_STARTED'),
            L_C.PGM_STATE_STOPPED: translate.gettext('PGM_STATE_STOPPED'),
            L_C.PGM_STATE_FINISHED: translate.gettext('PGM_STATE_FINISHED'),
            L_C.PGM_STATE_CANCELLED: translate.gettext('PGM_STATE_CANCELLED'),
            L_C.PGM_STATE_INTERRUPTED: translate.gettext('PGM_STATE_INTERRUPTED'),
            L_C.PGM_STATE_ERROR: translate.gettext('PGM_STATE_ERROR'),
            L_C.PGM_STATE_ERROR_CLEARED: translate.gettext('PGM_STATE_ERROR_CLEARED'),
            L_C.PGM_STATE_IDLE: translate.gettext('PGM_STATE_IDLE'),
            L_C.PGM_STATE_UNDEFINED: translate.gettext('PGM_STATE_UNDEFINED')}.get(code, translate.gettext('PGM_STATE_UNKNOWN'))

def get_execution_status_text(code, language='en'):
    """Translate status code of execution state to text
    See https://github.com/drunsinn/pyLSV2/issues/1

    :param int code: status code of execution status
    :param str language: optional. language code for translation of text
    :returns: readable text for execution state
    :rtype: str
    """
    locale_path = os.path.dirname(__file__) + '/locales'
    translate = gettext.translation(
        'message_text', localedir=locale_path, languages=[language], fallback=True)
    return {L_C.EXEC_STATE_MANUAL: translate.gettext('EXEC_STATE_MANUAL'),
            L_C.EXEC_STATE_MDI: translate.gettext('EXEC_STATE_MDI'),
            L_C.EXEC_STATE_PASS_REFERENCES: translate.gettext('EXEC_STATE_PASS_REFERENCES'),
            L_C.EXEC_STATE_SINGLE_STEP: translate.gettext('EXEC_STATE_SINGLE_STEP'),
            L_C.EXEC_STATE_AUTOMATIC: translate.gettext('EXEC_STATE_AUTOMATIC'),
            L_C.EXEC_STATE_UNDEFINED: translate.gettext('EXEC_STATE_UNDEFINED')}.get(code, translate.gettext('EXEC_STATE_UNKNOWN'))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" this is a demo script to show how to use lsv2 via an sshtunnel and paramiko
For thist to work you have to:
1. enable ssh connections on the control
2. create a ssh key pair on yyour computer
3. add the private key to one of the user accouts on the control (default is 'user')
4. make shure ssh is allowd throug the firewall on the control
5. install the python library 'sshtunnel'
6. edit this file and set address, user name and path to the key file
"""
import logging

import pyLSV2
from sshtunnel import SSHTunnelForwarder

logging.basicConfig(level=logging.INFO)

if __name__ == "__main__":
    address = '192.168.56.101'
    user_name = 'user'
    private_key_file = '<path to private key file>'
    lsv2_port = 19000

    print('Connecting to {}@{}:lsv2_port via ssh tunnel'.format(user_name, address, lsv2_port))
    ssh_forwarder = SSHTunnelForwarder(
        address, ssh_username=user_name, ssh_pkey=private_key_file, remote_bind_address=('127.0.0.1', lsv2_port))
    ssh_forwarder.start()
    print('SSH tunnel established. local port is {}'.format(
        ssh_forwarder.local_bind_port))

    print('Establish regular LSV2 connection via local port')
    lsv2 = pyLSV2.LSV2(
        '127.0.0.1', port=ssh_forwarder.local_bind_port, timeout=5, safe_mode=False)
    lsv2.connect()
    print('Connected to "{Control}" with NC Software "{NC_Version}"'.format(
        **lsv2.get_versions()))
    print('Close Connection')
    lsv2.disconnect()

    print('Close SSH tunnel')
    ssh_forwarder.stop()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""misc helper functions for pyLSV2"""
import struct
from datetime import datetime


def decode_system_parameters(result_set):
    """decode the result system parameter query

    :param tuple result_set: bytes returned by the system parameter query command R_PR
    :returns: dictionary with system parameter values
    :rtype: dict
    """
    message_length = len(result_set)
    info_list = list()
    # as per comment in eclipse plugin, there might be a difference between a programming station and a real machine
    if message_length == 120:
        info_list = struct.unpack('!14L8B8L2BH4B2L2HL', result_set)
    elif message_length == 124:
        info_list = struct.unpack('!14L8B8L2BH4B2L2HLL', result_set)
    else:
        raise ValueError('unexpected length {} of message content {}'.format(
            message_length, result_set))
    sys_par = dict()
    sys_par['Marker_Start'] = info_list[0]
    sys_par['Markers'] = info_list[1]
    sys_par['Input_Start'] = info_list[2]
    sys_par['Inputs'] = info_list[3]
    sys_par['Output_Start'] = info_list[4]
    sys_par['Outputs'] = info_list[5]
    sys_par['Counter_Start'] = info_list[6]
    sys_par['Counters'] = info_list[7]
    sys_par['Timer_Start'] = info_list[8]
    sys_par['Timers'] = info_list[9]
    sys_par['Word_Start'] = info_list[10]
    sys_par['Words'] = info_list[11]
    sys_par['String_Start'] = info_list[12]
    sys_par['Strings'] = info_list[13]
    sys_par['String_Length'] = info_list[14]
    sys_par['Input_Word_Start'] = info_list[22]
    sys_par['Input Words'] = info_list[23]
    sys_par['Output_Word_Start'] = info_list[24]
    sys_par['Output_Words'] = info_list[25]
    sys_par['LSV2_Version'] = info_list[30]
    sys_par['LSV2_Version_Flags'] = info_list[31]
    sys_par['Max_Block_Length'] = info_list[32]
    sys_par['HDH_Bin_Version'] = info_list[33]
    sys_par['HDH_Bin_Revision'] = info_list[34]
    sys_par['ISO_Bin_Version'] = info_list[35]
    sys_par['ISO_Bin_Revision'] = info_list[36]
    sys_par['HardwareVersion'] = info_list[37]
    sys_par['LSV2_Version_Flags_Ex'] = info_list[38]
    sys_par['Max_Trace_Line'] = info_list[39]
    sys_par['Scope_Channels'] = info_list[40]
    sys_par['PW_Encryption_Key'] = info_list[41]
    return sys_par


def decode_file_system_info(data_set):
    """decode result from file system entry

    :param tuple result_set: bytes returned by the system parameter query command R_FI or CR_DR
    :returns: dictionary with file system entry parameters
    :rtype: dict
    """
    file_info = dict()
    file_info['Size'] = struct.unpack('!L', data_set[:4])[0]
    file_info['Timestamp'] =  datetime.fromtimestamp(struct.unpack('!L', data_set[4:8])[0])

    arrtibutes = struct.unpack('!L', data_set[8:12])[0]
    file_info['Attributs'] = arrtibutes
    file_info['is_file'] = False
    file_info['is_directory'] = False
    file_info['is_drive'] = False

    if arrtibutes > 0:
        if bool(arrtibutes & 0x10):
            file_info['is_drive'] = True
        elif bool(arrtibutes & 0x20):
            file_info['is_directory'] = True
        else:
            file_info['is_file'] = True
        file_info['is_write_protected'] = bool(arrtibutes & 0x40)

    file_info['Name'] = data_set[12:].decode().strip('\x00').replace('\\', '/')

    return file_info

def decode_directory_info(data_set):
    """decode result from directory entry

    :param tuple result_set: bytes returned by the system parameter query command R_DI
    :returns: dictionary with file system entry parameters
    :rtype: dict
    """
    dir_info = dict()
    dir_info['Free Size'] = struct.unpack('!L', data_set[:4])[0]
    attribute_list = list()
    for i in range(4, len(data_set[4:132]), 4):
        attr = data_set[i:i + 4].decode().strip('\x00')
        if len(attr) > 0:
            attribute_list.append(attr)
    dir_info['Dir_Attributs'] = attribute_list

    dir_info['Attributs'] = struct.unpack('!32B', data_set[132:164])


    dir_info['Path'] = data_set[164:].decode().strip('\x00').replace('\\', '/')
    return dir_info

def decode_tool_information(data_set):
    """decode result from tool info

    :param tuple result_set: bytes returned by the system parameter query command R_RI for tool info
    :returns: dictionary with tool info values
    :rtype: dict
    """
    tool_info = dict()
    tool_info['Number'] = struct.unpack('!L', data_set[0:4])[0]
    tool_info['Index'] = struct.unpack('!H', data_set[4:6])[0]
    tool_info['Axis'] = {0: 'X', 1: 'Y', 2: 'Z'}.get(struct.unpack('!H', data_set[6:8])[0], 'unknown')
    tool_info['Length'] = struct.unpack('<d', data_set[8:16])[0]
    tool_info['Radius'] = struct.unpack('<d', data_set[16:24])[0]
    return tool_info
    
def decode_override_information(data_set):
    """decode result from override info

    :param tuple result_set: bytes returned by the system parameter query command R_RI for override info
    :returns: dictionary with override info values
    :rtype: dict
    """
    override_info = dict()
    override_info['Feed_override']=struct.unpack('!L', data_set[0:4])[0]/100
    override_info['Speed_override']=struct.unpack('!L', data_set[4:8])[0]/100
    override_info['Rapid_override']=struct.unpack('!L', data_set[8:12])[0]/100

    return override_info



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""low level communication functions for LSV2"""
import logging
import socket
import struct


class LLLSV2Com():
    """Implementation of the low level communication functions for sending and
       receiving LSV2 telegrams via TCP"""
    DEFAULT_PORT = 19000  # Default port for LSV2 on control side
    DEFAULT_BUFFER_SIZE = 256

    def __init__(self, hostname, port=19000, timeout=15.0):
        """Set connection parameters

        :param str hostname: ip or hostname of control.
        :param int port: port number, defaults to 19000.
        :param float timeout: number of seconds for time out of connection.
        """
        try:
            self._host_ip = socket.gethostbyname(hostname)
        except socket.gaierror:
            logging.error('there was An error resolving the host')
            raise

        self._port = port

        try:
            self._tcpsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self._tcpsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self._tcpsock.settimeout(timeout)
        except socket.error as err:
            logging.error('socket creation failed with error %s', err)
            raise
        self._is_connected = False

        logging.debug(
            'Socket successfully created, host %s was resolved to IP %s', hostname, self._host_ip)

    def connect(self):
        """Establish connection to control

        :raise: Exception if connection times out.
        :rtype: None
        """
        try:
            self._tcpsock.connect((self._host_ip, self._port))
        except socket.timeout:
            logging.error('could not connect to control')
            raise
        self._is_connected = True
        logging.debug('Connected to host %s at port %s',
                      self._host_ip, self._port)

    def disconnect(self):
        """Close connection

        :raise: Exception if connection times out.
        :rtype: None
        """
        try:
            self._tcpsock.close()
        except socket.timeout:
            logging.error('error while closing socket')
            raise
        self._is_connected = False
        logging.debug('Connection to %s closed', self._host_ip)

    def telegram(self, command, payload=None, buffer_size=0, wait_for_response=True):
        """Send LSV2 telegram and receive response if necessary.

        :param str command: command string.
        :param byte array payload: command payload.
        :param int buffer_size: size of message buffer used by control.
        :param bool wait_for_response: switch for waiting for response from control.
        :raise: Exception if connection is not already open or error during transmission.
        :return: response message and content
        :rtype: list
        """
        if self._is_connected is False:
            raise Exception('connection is not open!')

        if payload is None:
            payload_length = 0
        else:
            payload_length = len(payload)

        if buffer_size < 8:
            buffer_size = LLLSV2Com.DEFAULT_BUFFER_SIZE
            logging.warning(
                'size of receive buffer to small, set to default of %d bytes', LLLSV2Com.DEFAULT_BUFFER_SIZE)

        telegram = bytearray()
        # L -> unsigned long -> 32 bit
        telegram.extend(struct.pack('!L', payload_length))
        telegram.extend(map(ord, command))
        if payload is not None:
            telegram.extend(payload)
        logging.debug('telegram to transmit: command %s payload length %d bytes data: %s',
                      command, payload_length, telegram)
        if len(telegram) >= buffer_size:
            raise OverflowError('telegram to long for set current buffer size: %d >= %d' % (
                len(telegram), buffer_size))

        response = None
        try:
            self._tcpsock.send(telegram)
            if wait_for_response:
                response = self._tcpsock.recv(buffer_size)
        except:
            logging.error(
                'somthing went wrong while waiting for new data to arrive, buffer was set to %d', buffer_size)
            raise

        if response is not None:
            response_length = struct.unpack('!L', response[0:4])[
                0]  # read 4 bytes for response length
            response_command = response[4:8].decode(
                'utf-8', 'ignore')  # read 4 bytes for response type
        else:
            response_length = 0
            response_command = None

        if response_length > 0:
            response_content = bytearray(response[8:])
            while len(response_content) < response_length:
                logging.debug('waiting for more data to arrive, %d bytes missing', len(
                    response_content) < response_length)
                try:
                    response_content.extend(self._tcpsock.recv(
                        response_length-len(response[8:])))
                except:
                    logging.error(
                        'somthing went wrong while waiting for more data to arrive')
                    raise
        else:
            response_content = None

        return response_command, response_content




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import pathlib
from pyLSV2 import TableReader

logging.basicConfig(level=logging.DEBUG)

if __name__ == "__main__":

    data_dir = pathlib.Path('../data/')

    for subfolder in ('table_itnc530', 'table_pilot640', 'table_tnc640'):

        current_folder = data_dir.joinpath(subfolder)

        for table_path in current_folder.glob('*.*'):
            tr = TableReader()
            tr.parse_table(table_path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import re
from pathlib import Path


class TableReader():
    """generic parser for table files commonly used by TNC, iTNC, CNCPILOT and MANUALplus controls"""

    def __init__(self):
        """init object variables logging"""
        logging.getLogger(__name__).addHandler(logging.NullHandler())
        self.name = None
        self.suffix = None
        self.version = None
        self.has_unit = False
        self.is_metric = False

    def parse_table(self, table_path):
        """Parse a file of one of the common table formats

        :param str or Path table_path: Path to the table file

        :returns: list od dictionarys. key is the column name, value the content of the table cell
        :rtype: list
        """
        self.name = None
        self.suffix = None
        self.version = None
        self.has_unit = False
        self.is_metric = False

        table_file = Path(table_path)
        if not table_file.is_file():
            raise FileNotFoundError('Could not open file %s' % table_path)

        with table_file.open(mode='r') as tfp:
            header_line = tfp.readline().strip()
            logging.debug('Checking line for header: %s', header_line)
            header = re.match(
                r"^BEGIN (?P<name>[a-zA-Z_ 0-9]*)\.(?P<suffix>[A-Za-z0-9]{1,4})(?P<unit> MM| INCH)?(?: Version: \'Update:(?P<version>\d+\.\d+)\')?$", header_line)

            if header is None:
                raise Exception(
                    'File has wrong format: incorrect header for file %s' % table_path)

            self.name = header.group('name').strip()
            self.suffix = header.group('suffix')
            self.version = header.group('version')

            if header.group('unit') is not None:
                self.has_unit = True
                if 'MM' in header.group('unit'):
                    self.is_metric = True
                    logging.debug(
                        'Header Infomation for file "%s" Name "%s", file is metric, Version: "%s"', table_file, self.name, self.version)
                else:
                    self.is_metric = False
                    logging.debug(
                        'Header Infomation for file "%s" Name "%s", file is inch, Version: "%s"', table_file, self.name, self.version)
            else:
                self.has_unit = False
                self.is_metric = False
                logging.debug('Header Infomation for file "%s" Name "%s", file has no units, Version: "%s"',
                              table_file, self.name, self.version)

            next_line = tfp.readline()
            if '#STRUCTBEGIN' in next_line or 'TableDescription' in next_line:
                in_preamble = True
                next_line = tfp.readline()
                while in_preamble:
                    if next_line.startswith('#'):
                        in_preamble = False
                    elif next_line.startswith(')'):
                        in_preamble = False
                    else:
                        next_line = tfp.readline()
                next_line = tfp.readline()

            column_header = next_line
            column_list = list()
            column_pattern = re.compile(r"([A-Za-z-12_:\.]+)(?:\s+)")
            for column_match in column_pattern.finditer(column_header):
                column_list.append({'start': column_match.start(
                ), 'end': column_match.end()-1, 'name': column_match.group().strip()})
                
            logging.debug('Found %d columns', len(column_list))

            table_content = list()

            for line in tfp.readlines():
                if line.startswith('[END]'):
                    break

                table_entry = dict()

                for column in column_list:
                    table_entry[column['name']
                                ] = line[column['start']:column['end']].strip()
                table_content.append(table_entry)

            logging.debug('Found %d entrys', len(table_content))
        return table_content

    @staticmethod
    def format_entry_float(str_value):
        """convert the string value o a table cell to float value"""
        str_value = str_value.strip()
        if str_value == '-' or len(str_value) == 0:
            return None

        return float(str_value)

    @staticmethod
    def format_entry_int(str_value):
        """convert the string value o a table cell to int value"""
        str_value = str_value.strip()
        if str_value == '-' or len(str_value) == 0:
            return None

        return int(str_value)

    @staticmethod
    def format_entry_bool(str_value):
        """convert the string value o a table cell to boolean value"""
        str_value = str_value.strip()
        if len(str_value) == 0:
            return None
        elif str_value == '1':
            return True

        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import string
import time
import struct
import pyLSV2


def test_command(lsv2, command_string, payload=None):
    """check commands for validity"""
    response, content = lsv2._llcom.telegram(
        command_string, payload, buffer_size=lsv2._buffer_size)
    if content is None:
        response_length = -1
    else:
        response_length = len(content)
    if response in lsv2.RESPONSE_T_ER:
        if len(content) == 2:
            byte_1, byte_2, = struct.unpack('!BB', content)
            error_text = lsv2.get_error_text(byte_1, byte_2)
        else:
            error_text = 'Unknown Error Number {}'.format(content)
    else:
        error_text = 'NONE'
    return 'sent {} payload {} received {} message_length {} content {} error text : {}'.format(command_string, payload, response, response_length, content, error_text)


if __name__ == '__main__':
    logging.basicConfig(format='%(levelname)s : %(message)s', level=logging.ERROR)
    
    lsv2 = pyLSV2.LSV2('192.168.56.103', safe_mode=False)

    lsv2.connect()
    lsv2.login(login=pyLSV2.LOGIN_INSPECT)
    lsv2.login(login=pyLSV2.LOGIN_DNC)
    lsv2.login(login=pyLSV2.LOGIN_INSPECT)
    lsv2.login(login=pyLSV2.LOGIN_DIAG)
    lsv2.login(login=pyLSV2.LOGIN_PLCDEBUG)
    lsv2.login(login=pyLSV2.LOGIN_FILETRANSFER)
    lsv2.login(login=pyLSV2.LOGIN_MONITOR)
    lsv2.login(login=pyLSV2.LOGIN_DSP)
    lsv2.login(login=pyLSV2.LOGIN_DNC)
    lsv2.login(login=pyLSV2.LOGIN_SCOPE)
    lsv2.login(login=pyLSV2.LOGIN_FILEPLC, password='<PLC_PASSWORD>')

    # base_string = 'A_'
    # with open('./out.txt', 'a') as fp:
    #     for first_letter in list(string.ascii_uppercase)[:]:
    #         for second_letter in list(string.ascii_uppercase)[:]:
    #             cmd = base_string + first_letter + second_letter
    #             fp.write(lsv2._test_command(command_string=cmd, payload=None) + '\n')
    #             time.sleep(0.5)

    with open('./out.txt', 'a') as fp:
        for i in range (0, 0xFFFFFFFF):
            fp.write(test_command(lsv2, command_string='R_MB', payload=struct.pack('!L', i)) + '\n')


    lsv2.disconnect()
